'use strict';

let calculator = {
    index: document.getElementById("calculator"),
    calculators: [],
    account: [],
    settings: {
        defaults: {
            buyer: 'individual',
            complexity: 'medium',
        }
    },
    init() {
        // Инициализация калькулятора

        // !!!!!!!!!!!!!!!!! РАЗОБРАТЬСЯ С ПОРЯДКОМ ВЫПОЛНЕНИЯ

        this.generate.buyer(this.settings.defaults.buyer)
            .then(this.generate.complexity(this.settings.defaults.complexity)
                .then(this.generate.menu()
                    .then(this.authenticate(cookie.read('id'))
                        .then(success => {
                            // Запись данных аккаунта
                            this.account = success;

                            if (this.account !== undefined && typeof this.account === 'object' && this.account.permissions !== undefined) {
                                // Найден аккаунт

                                if (this.account.permissions.calculate == 1) {
                                    // Разрешено использовать калькулятор

                                    this.generate.result();
                                }
                            }
                        }
                        )
                    )
                )
            );

        console.log('[КАЛЬКУЛЯТОР] Инициализирован');
    },
    authenticate(id) {
        // Запрос и генерация HTML с данными о типе покупателя (юр. лицо и физ. лицо)'

        return fetch('/account/data?id=' + id, {
            method: "POST",
            headers: { "content-type": "application/x-www-form-urlencoded" }
        }).then((response) => {
            if (response.status === 200) {
                return response.json().then(
                    success => {
                        console.log('[КАЛЬКУЛЯТОР] Загружены данные пользователя: ' + id);

                        return success;
                    },
                    error => {
                        console.log('[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось загрузить данные пользователя: ' + id);
                    });
            }
        });
    },
    calculate() {
        // Запрос и генерация HTML с данными о рассчете со всех калькуляторов

        // Инициализация буферов вывода
        let expenses, income, profit;

        // Инициализация буфера запроса
        let query = {};

        for (const number in this.calculators) {
            // Перебор калькуляторов

            // Инициализация буфера запроса для нового калькулятора
            query[number] = {};

            // Инициализация типа калькулятора
            query[number]['type'] = this.calculators[number].getAttribute('data-calculator');

            for (const buyer of this.index.querySelectorAll('input[name="buyer"]')) {
                // Перебор полей с параметрами типа заказчика

                if (buyer.checked) {
                    // Найдено выбранное поле

                    // Запись в буфер запроса
                    query[number]['buyer'] = buyer.value;
                }
            }

            for (const complexity of this.index.querySelectorAll('input[name="complexity"]')) {
                // Перебор полей с параметрами сложности

                if (complexity.checked) {
                    // Найдено выбранное поле

                    // Запись в буфер запроса
                    query[number]['complexity'] = complexity.value;
                }
            }

            for (const field of this.calculators[number].querySelectorAll('[data-calculator-parameter]')) {
                // Перебор полей с параметрами

                // Запись в буфер запроса
                query[number][field.getAttribute('data-calculator-parameter')] = field.value ?? field.options[field.selectedIndex].text;
            }

            // Сортировка
            query[number] = this.sort[query[number]['type']](query[number]);
        }

        fetch('/calculator/calculate', {
            method: "POST",
            headers: { "content-type": "application/json" },
            body: JSON.stringify(query)
        }).then((response) => {
            if (response.status === 200) {
                return response.json().then(
                    success => {
                        console.log('[КАЛЬКУЛЯТОР] Сгенерирован результат');

                        return success;
                    },
                    error => {
                        console.log('[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось сгенерировать результат');
                    });
            }
        });

        let result = document.getElementById("result");

        if (result === null) {
            // Не найден элемент с данными рассчета

            // Инициализия
            this.generate.result();
        }
    },
    generate: {
        buyer(value = 'individual') {
            // Запрос и генерация HTML с данными о типе покупателя (юр. лицо и физ. лицо)

            return fetch('/calculator/generate/buyer?value=' + value, {
                method: "POST",
                headers: { "content-type": "application/x-www-form-urlencoded" }
            }).then((response) => {
                if (response.status === 200) {
                    response.text().then(
                        success => {
                            calculator.index.insertAdjacentHTML('beforeend', success);

                            console.log('[КАЛЬКУЛЯТОР] Загружен элемент с кнопками выбора типа покупателя');
                        },
                        error => {
                            console.log('[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось загрузить элемент с кнопками выбора типа покупателя');
                        });
                }
            });
        },
        complexity(value = 'medium') {
            // Запрос и генерация HTML с данными о сложности работы

            return fetch('/calculator/generate/complexity?value=' + value, {
                method: "POST",
                headers: { "content-type": "application/x-www-form-urlencoded" }
            }).then((response) => {
                if (response.status === 200) {
                    response.text().then(
                        success => {
                            calculator.index.insertAdjacentHTML('beforeend', success);

                            console.log('[КАЛЬКУЛЯТОР] Загружен элемент с кнопками выбора сложности');
                        },
                        error => {
                            console.log('[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось загрузить элемент с кнопками выбора сложности');
                        });
                }
            });
        },
        menu() {
            // Запрос и генерация HTML с кнопками добавления калькулятора

            return fetch('/calculator/generate/menu', {
                method: "POST",
                headers: { "content-type": "application/x-www-form-urlencoded" }
            }).then((response) => {
                if (response.status === 200) {
                    response.text().then(
                        success => {
                            calculator.index.insertAdjacentHTML('beforeend', success);

                            console.log('[КАЛЬКУЛЯТОР] Загружен элемент с кнопками добавления калькулятора');
                        },
                        error => {
                            console.log('[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось загрузить элемент с кнопками добавления калькулятора');
                        });
                }
            });
        },
        result() {
            // Запрос и генерация HTML с данными о результате калькуляции

            return fetch('/calculator/generate/result', {
                method: "POST",
                headers: { "content-type": "application/x-www-form-urlencoded" }
            }).then((response) => {
                if (response.status === 200) {
                    response.text().then(
                        success => {
                            calculator.index.insertAdjacentHTML('beforeend', success);

                            console.log('[КАЛЬКУЛЯТОР] Загружен элемент с данными о результате калькуляции');
                        },
                        error => {
                            console.log('[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось загрузить элемент с данными о результате калькуляции');
                        });
                }
            });
        },
        divider(element, position) {
            // Запрос и генерация HTML с данными о результате калькуляции

            return fetch('/calculator/generate/divider', {
                method: "POST",
                headers: { "content-type": "application/x-www-form-urlencoded" }
            }).then((response) => {
                if (response.status === 200) {
                    response.text().then(
                        success => {
                            if (element === undefined || position === undefined) {
                                // Не задан элемент и позиция добавляемого разделителя

                                // Запись разделителя в конце калькулятора
                                calculator.index.insertAdjacentHTML('beforeend', success);
                            } else {
                                // Задан элемент и позиция добавляемого разделителя

                                // Запись разделителя по заданным параметрам
                                element.insertAdjacentHTML(position, success);
                            }

                            console.log('[КАЛЬКУЛЯТОР] Загружен разделитель');
                        },
                        error => {
                            console.log('[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось загрузить разделитель');
                        });
                }
            });
        },
        calculators: {
            laser() {
                // Запрос и генерация HTML с калькулятором лазерной резки

                // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ИЗБАВИТЬСЯ ОТ ТАЙМАУТОВ

                return fetch('/calculator/generate/laser', {
                    method: "POST",
                    headers: { "content-type": "application/x-www-form-urlencoded" }
                }).then((response) => {
                    if (response.status === 200) {
                        response.text().then(
                            success => {
                                // Поиск последнего калькулятора
                                let last = calculator.calculators[calculator.calculators.length - 1];

                                if (last !== undefined && last !== null) {
                                    // Найден калькулятор

                                    // Инициализация разделителя перед меню
                                    calculator.generate.divider(last, 'afterend').then(
                                        divider => {
                                            // Запись калькулятора после последнего калькулятора
                                            last.insertAdjacentHTML('afterend', success);

                                            // Поиск калькуляторов
                                            let calculators = calculator.index.querySelectorAll('section[data-calculator]');

                                            // Запись в реестр последнего калькулятора (подразумевается, что он новый и только что был записан)
                                            calculator.calculators.push(calculators[calculators.length - 1]);

                                            // Запись в журнал
                                            console.log('[КАЛЬКУЛЯТОР] Инициализирован калькулятор лазерной резки');
                                        }
                                    );
                                } else {
                                    // Не найден калькулятор

                                    // Поиск меню
                                    let menu = document.getElementById("menu");

                                    if (menu !== null) {
                                        // Найдено меню

                                        // Инициализация разделителя перед меню
                                        calculator.generate.divider(menu, 'beforebegin').then(
                                            divider => {
                                                // Запись калькулятора перед меню
                                                menu.insertAdjacentHTML('beforebegin', success);

                                                // Поиск калькуляторов
                                                let calculators = calculator.index.querySelectorAll('section[data-calculator]');

                                                // Запись в реестр последнего калькулятора (подразумевается, что он новый и только что был записан)
                                                calculator.calculators.push(calculators[calculators.length - 1]);

                                                // Запись в журнал
                                                console.log('[КАЛЬКУЛЯТОР] Инициализирован калькулятор лазерной резки');
                                            }
                                        );
                                    } else {
                                        // Не найдено меню

                                        // Поиск результатов калькуляции
                                        let result = document.getElementById("result");

                                        if (result !== null) {
                                            // Найден элемент с результатами калькуляции

                                            // Инициализация разделителя перед меню
                                            calculator.generate.divider(result, 'beforebegin').then(
                                                divider => {
                                                    // Запись калькулятора перед элементом с результатами калькуляции
                                                    result.insertAdjacentHTML('beforebegin', success);

                                                    // Поиск калькуляторов
                                                    let calculators = calculator.index.querySelectorAll('section[data-calculator]');

                                                    // Запись в реестр последнего калькулятора (подразумевается, что он новый и только что был записан)
                                                    calculator.calculators.push(calculators[calculators.length - 1]);

                                                    // Запись в журнал
                                                    console.log('[КАЛЬКУЛЯТОР] Инициализирован калькулятор лазерной резки');
                                                }
                                            );
                                        } else {
                                            // Не найден элемент с результатами калькуляции

                                            // Инициализация разделителя перед меню
                                            calculator.generate.divider().then(
                                                divider => {
                                                    // Запись калькулятора в конце калькулятора
                                                    calculator.index.insertAdjacentHTML('beforeend', success);

                                                    // Поиск калькуляторов
                                                    let calculators = calculator.index.querySelectorAll('section[data-calculator]');

                                                    // Запись в реестр последнего калькулятора (подразумевается, что он новый и только что был записан)
                                                    calculator.calculators.push(calculators[calculators.length - 1]);

                                                    // Запись в журнал
                                                    console.log('[КАЛЬКУЛЯТОР] Инициализирован калькулятор лазерной резки');
                                                }
                                            );
                                        }
                                    }
                                }
                            },
                            error => {
                                // Запись в журнал
                                console.log('[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось инициализировать калькулятор лазерной резки');
                            });
                    }
                });
            }
        }
    },
    sort: {
        laser(parameters) {
            // Сортировка параметров для отправки на сервер (динамически вызывается функция-обработчик)

            return {
                type: 'laser',
                buyer: parameters['buyer'] ?? null,
                complexity: parameters['complexity'] ?? null,
                width: parameters['width'] ?? null,
                height: parameters['width'] ?? null,
                length: parameters['length'] ?? null,
                amount: parameters['amount'] ?? null,
                metal: parameters['metal'] ?? null,
                holes: parameters['holes'] ?? null,
                diameter: parameters['diameter'] ?? null,
            };
        }
    }
};
